@@ -19,19 +19,17 @@
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "engine_p.h"
+#include "engine.h"
 
-#include "entry.h"
-#include "core/installation_p.h"
-#include "core/xmlloader_p.h"
-#include "ui/imageloader_p.h"
+#include "../entry.h"
+#include "installation.h"
+#include "xmlloader.h"
+#include "imageloader_p.h"
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
-#include <QDebug>
+#include <knewstuffcore_debug.h>
 #include <klocalizedstring.h>
-
-#include <kio/job.h>
 #include <QDesktopServices>
 
 #include <QtCore/QTimer>
@@ -41,7 +39,6 @@
 
 #if defined(Q_OS_WIN)
 #include <windows.h>
-#define _WIN32_IE 0x0500
 #include <shlobj.h>
 #endif
 
@@ -50,30 +47,32 @@
 #include <qstandardpaths.h>
 
 // own
-#include "attica/atticaprovider_p.h"
-#include "core/cache_p.h"
-#include "staticxml/staticxmlprovider_p.h"
+#include "../attica/atticaprovider_p.h"
+#include "cache.h"
+#include "../staticxml/staticxmlprovider_p.h"
 
-using namespace KNS3;
+using namespace KNSCore;
 
 Engine::Engine(QObject *parent)
     : QObject(parent)
-    , m_initialized(false)
     , m_installation(new Installation)
     , m_cache(0)
     , m_searchTimer(new QTimer)
+    , m_atticaProviderManager(0)
     , m_currentPage(-1)
     , m_pageSize(20)
     , m_numDataJobs(0)
     , m_numPictureJobs(0)
     , m_numInstallJobs(0)
-    , m_atticaProviderManager(0)
+    , m_initialized(false)
 {
     m_searchTimer->setSingleShot(true);
     m_searchTimer->setInterval(1000);
-    connect(m_searchTimer, SIGNAL(timeout()), SLOT(slotSearchTimerExpired()));
-    connect(m_installation, SIGNAL(signalInstallationFinished()), this, SLOT(slotInstallationFinished()));
-    connect(m_installation, SIGNAL(signalInstallationFailed(QString)), this, SLOT(slotInstallationFailed(QString)));
+    connect(m_searchTimer, &QTimer::timeout, this, &Engine::slotSearchTimerExpired);
+    connect(m_installation, &Installation::signalInstallationFinished, this, &Engine::slotInstallationFinished);
+    connect(m_installation, &Installation::signalInstallationFailed, this, &Engine::slotInstallationFailed);
+    connect(m_installation, &Installation::signalInformation, this, &Engine::signalMessage);
+    connect(m_installation, &Installation::signalError, this, &Engine::signalError);
 
 }
 
@@ -89,7 +88,7 @@ Engine::~Engine()
 
 bool Engine::init(const QString &configfile)
 {
-    // qDebug() << "Initializing KNS3::Engine from '" << configfile << "'";
+    qCDebug(KNEWSTUFFCORE) << "Initializing KNSCore::Engine from '" << configfile << "'";
 
     emit signalBusy(i18n("Initializing"));
 
@@ -99,22 +98,13 @@ bool Engine::init(const QString &configfile)
         qCritical() << "No knsrc file named '" << configfile << "' was found." << endl;
         return false;
     }
-    // FIXME: accessMode() doesn't return NoAccess for non-existing files
-    // - bug in kdecore?
-    // - this needs to be looked at again until KConfig backend changes for KDE 4
-    // the check below is a workaround
-    if (QStandardPaths::locate(QStandardPaths::GenericConfigLocation, configfile).isEmpty()) {
-        emit signalError(i18n("Configuration file not found: \"%1\"", configfile));
-        qCritical() << "No knsrc file named '" << configfile << "' was found." << endl;
-        return false;
-    }
 
     KConfigGroup group;
     if (conf.hasGroup("KNewStuff3")) {
-        // qDebug() << "Loading KNewStuff3 config: " << configfile;
+        qCDebug(KNEWSTUFFCORE) << "Loading KNewStuff3 config: " << configfile;
         group = conf.group("KNewStuff3");
     } else if (conf.hasGroup("KNewStuff2")) {
-        // qDebug() << "Loading KNewStuff2 config: " << configfile;
+        qCDebug(KNEWSTUFFCORE) << "Loading KNewStuff2 config: " << configfile;
         group = conf.group("KNewStuff2");
     } else {
         emit signalError(i18n("Configuration file is invalid: \"%1\"", configfile));
@@ -124,19 +114,19 @@ bool Engine::init(const QString &configfile)
 
     m_categories = group.readEntry("Categories", QStringList());
 
-    // qDebug() << "Categories: " << m_categories;
+    qCDebug(KNEWSTUFFCORE) << "Categories: " << m_categories;
     m_providerFileUrl = group.readEntry("ProvidersUrl", QString());
-    m_applicationName = QFileInfo(QStandardPaths::locate(QStandardPaths::GenericConfigLocation, configfile)).baseName() + ':';
 
+    const QString configFileName = QFileInfo(QDir::isAbsolutePath(configfile) ? configfile : QStandardPaths::locate(QStandardPaths::GenericConfigLocation, configfile)).baseName();
     // let installation read install specific config
     if (!m_installation->readConfig(group)) {
         return false;
     }
 
-    connect(m_installation, SIGNAL(signalEntryChanged(KNS3::EntryInternal)), SLOT(slotEntryChanged(KNS3::EntryInternal)));
+    connect(m_installation, &Installation::signalEntryChanged, this, &Engine::slotEntryChanged);
 
-    m_cache = Cache::getCache(m_applicationName.split(':')[0]);
-    connect(this, SIGNAL(signalEntryChanged(KNS3::EntryInternal)), m_cache.data(), SLOT(registerChangedEntry(KNS3::EntryInternal)));
+    m_cache = Cache::getCache(configFileName);
+    connect(this, &Engine::signalEntryChanged, m_cache.data(), &Cache::registerChangedEntry);
     m_cache->readRegistry();
 
     m_initialized = true;
@@ -161,17 +151,18 @@ void Engine::loadProviders()
 {
     if (m_providerFileUrl.isEmpty()) {
         // it would be nicer to move the attica stuff into its own class
-        // qDebug() << "Using OCS default providers";
-        Attica::ProviderManager *m_atticaProviderManager = new Attica::ProviderManager;
-        connect(m_atticaProviderManager, SIGNAL(providerAdded(Attica::Provider)), this, SLOT(atticaProviderLoaded(Attica::Provider)));
+        qCDebug(KNEWSTUFFCORE) << "Using OCS default providers";
+        delete m_atticaProviderManager;
+        m_atticaProviderManager = new Attica::ProviderManager;
+        connect(m_atticaProviderManager, &Attica::ProviderManager::providerAdded, this, &Engine::atticaProviderLoaded);
         m_atticaProviderManager->loadDefaultProviders();
     } else {
-        // qDebug() << "loading providers from " << m_providerFileUrl;
+        qCDebug(KNEWSTUFFCORE) << "loading providers from " << m_providerFileUrl;
         emit signalBusy(i18n("Loading provider information"));
 
         XmlLoader *loader = new XmlLoader(this);
-        connect(loader, SIGNAL(signalLoaded(QDomDocument)), SLOT(slotProviderFileLoaded(QDomDocument)));
-        connect(loader, SIGNAL(signalFailed()), SLOT(slotProvidersFailed()));
+        connect(loader, &XmlLoader::signalLoaded, this, &Engine::slotProviderFileLoaded);
+        connect(loader, &XmlLoader::signalFailed, this, &Engine::slotProvidersFailed);
 
         loader->load(QUrl(m_providerFileUrl));
     }
@@ -179,30 +170,30 @@ void Engine::loadProviders()
 
 void Engine::slotProviderFileLoaded(const QDomDocument &doc)
 {
-    // qDebug() << "slotProvidersLoaded";
+    qCDebug(KNEWSTUFFCORE) << "slotProvidersLoaded";
 
     bool isAtticaProviderFile = false;
 
     // get each provider element, and create a provider object from it
     QDomElement providers = doc.documentElement();
 
-    if (providers.tagName() == "providers") {
+    if (providers.tagName() == QLatin1String("providers")) {
         isAtticaProviderFile = true;
-    } else if (providers.tagName() != "ghnsproviders" && providers.tagName() != "knewstuffproviders") {
+    } else if (providers.tagName() != QLatin1String("ghnsproviders") && providers.tagName() != QLatin1String("knewstuffproviders")) {
         qWarning() << "No document in providers.xml.";
         emit signalError(i18n("Could not load get hot new stuff providers from file: %1", m_providerFileUrl));
         return;
     }
 
-    QDomElement n = providers.firstChildElement("provider");
+    QDomElement n = providers.firstChildElement(QStringLiteral("provider"));
     while (!n.isNull()) {
-        // qDebug() << "Provider attributes: " << n.attribute("type");
+        qCDebug(KNEWSTUFFCORE) << "Provider attributes: " << n.attribute(QStringLiteral("type"));
 
-        QSharedPointer<KNS3::Provider> provider;
-        if (isAtticaProviderFile || n.attribute("type").toLower() == "rest") {
-            provider = QSharedPointer<KNS3::Provider> (new AtticaProvider(m_categories));
+        QSharedPointer<KNSCore::Provider> provider;
+        if (isAtticaProviderFile || n.attribute(QStringLiteral("type")).toLower() == QLatin1String("rest")) {
+            provider = QSharedPointer<KNSCore::Provider> (new AtticaProvider(m_categories));
         } else {
-            provider = QSharedPointer<KNS3::Provider> (new StaticXmlProvider);
+            provider = QSharedPointer<KNSCore::Provider> (new StaticXmlProvider);
         }
 
         if (provider->setProviderXML(n)) {
@@ -217,25 +208,27 @@ void Engine::slotProviderFileLoaded(const QDomDocument &doc)
 
 void Engine::atticaProviderLoaded(const Attica::Provider &atticaProvider)
 {
+    qCDebug(KNEWSTUFFCORE) << "atticaProviderLoaded called";
     if (!atticaProvider.hasContentService()) {
-        // qDebug() << "Found provider: " << atticaProvider.baseUrl() << " but it does not support content";
+        qCDebug(KNEWSTUFFCORE) << "Found provider: " << atticaProvider.baseUrl() << " but it does not support content";
         return;
     }
-    QSharedPointer<KNS3::Provider> provider =
-        QSharedPointer<KNS3::Provider> (new AtticaProvider(atticaProvider, m_categories));
+    QSharedPointer<KNSCore::Provider> provider =
+        QSharedPointer<KNSCore::Provider> (new AtticaProvider(atticaProvider, m_categories));
     addProvider(provider);
 }
 
-void Engine::addProvider(QSharedPointer<KNS3::Provider> provider)
+void Engine::addProvider(QSharedPointer<KNSCore::Provider> provider)
 {
+    qCDebug(KNEWSTUFFCORE) << "Engine addProvider called with provider with id " << provider->id();
     m_providers.insert(provider->id(), provider);
-    connect(provider.data(), SIGNAL(providerInitialized(KNS3::Provider*)), SLOT(providerInitialized(KNS3::Provider*)));
-    connect(provider.data(), SIGNAL(loadingFinished(KNS3::Provider::SearchRequest,KNS3::EntryInternal::List)),
-            SLOT(slotEntriesLoaded(KNS3::Provider::SearchRequest,KNS3::EntryInternal::List)));
-    connect(provider.data(), SIGNAL(entryDetailsLoaded(KNS3::EntryInternal)), SLOT(slotEntryDetailsLoaded(KNS3::EntryInternal)));
-    connect(provider.data(), SIGNAL(payloadLinkLoaded(KNS3::EntryInternal)), SLOT(downloadLinkLoaded(KNS3::EntryInternal)));
-    connect(provider.data(), SIGNAL(signalError(QString)), this, SIGNAL(signalError(QString)));
-    connect(provider.data(), SIGNAL(signalInformation(QString)), this, SIGNAL(signalIdle(QString)));
+    connect(provider.data(), &Provider::providerInitialized, this, &Engine::providerInitialized);
+    connect(provider.data(), SIGNAL(loadingFinished(KNSCore::Provider::SearchRequest,KNSCore::EntryInternal::List)),
+            SLOT(slotEntriesLoaded(KNSCore::Provider::SearchRequest,KNSCore::EntryInternal::List)));
+    connect(provider.data(), &Provider::entryDetailsLoaded, this, &Engine::slotEntryDetailsLoaded);
+    connect(provider.data(), &Provider::payloadLinkLoaded, this, &Engine::downloadLinkLoaded);
+    connect(provider.data(), &Provider::signalError, this, &Engine::signalError);
+    connect(provider.data(), &Provider::signalInformation, this, &Engine::signalIdle);
 }
 
 void Engine::providerJobStarted(KJob *job)
@@ -250,11 +243,11 @@ void Engine::slotProvidersFailed()
 
 void Engine::providerInitialized(Provider *p)
 {
-    // qDebug() << "providerInitialized" << p->name();
+    qCDebug(KNEWSTUFFCORE) << "providerInitialized" << p->name();
     p->setCachedEntries(m_cache->registryForProvider(p->id()));
     updateStatus();
 
-    foreach (const QSharedPointer<KNS3::Provider> &p, m_providers) {
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
         if (!p->isInitialized()) {
             return;
         }
@@ -262,12 +255,12 @@ void Engine::providerInitialized(Provider *p)
     emit signalProvidersLoaded();
 }
 
-void Engine::slotEntriesLoaded(const KNS3::Provider::SearchRequest &request, KNS3::EntryInternal::List entries)
+void Engine::slotEntriesLoaded(const KNSCore::Provider::SearchRequest &request, KNSCore::EntryInternal::List entries)
 {
     m_currentPage = qMax<int>(request.page, m_currentPage);
-    // qDebug() << "loaded page " << request.page << "current page" << m_currentPage;
+    qCDebug(KNEWSTUFFCORE) << "loaded page " << request.page << "current page" << m_currentPage;
 
-    if (request.sortMode == Provider::Updates) {
+    if (request.filter == Provider::Updates) {
         emit signalUpdateableEntriesLoaded(entries);
     } else {
         m_cache->insertRequest(request, entries);
@@ -285,16 +278,16 @@ void Engine::reloadEntries()
     m_currentRequest.page = 0;
     m_numDataJobs = 0;
 
-    foreach (const QSharedPointer<KNS3::Provider> &p, m_providers) {
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
         if (p->isInitialized()) {
-            if (m_currentRequest.sortMode == Provider::Installed) {
+            if (m_currentRequest.filter == Provider::Installed) {
                 // when asking for installed entries, never use the cache
                 p->loadEntries(m_currentRequest);
             } else {
                 // take entries from cache until there are no more
                 EntryInternal::List cache = m_cache->requestFromCache(m_currentRequest);
                 while (!cache.isEmpty()) {
-                    // qDebug() << "From cache";
+                    qCDebug(KNEWSTUFFCORE) << "From cache";
                     emit signalEntriesLoaded(cache);
 
                     m_currentPage = m_currentRequest.page;
@@ -309,7 +302,7 @@ void Engine::reloadEntries()
 
                 // if the cache was empty, request data from provider
                 if (m_currentPage == -1) {
-                    // qDebug() << "From provider";
+                    qCDebug(KNEWSTUFFCORE) << "From provider";
                     p->loadEntries(m_currentRequest);
 
                     ++m_numDataJobs;
@@ -335,6 +328,28 @@ void Engine::setSortMode(Provider::SortMode mode)
     reloadEntries();
 }
 
+void KNSCore::Engine::setFilter(Provider::Filter filter)
+{
+    if (m_currentRequest.filter != filter) {
+        m_currentRequest.page = -1;
+    }
+    m_currentRequest.filter = filter;
+    reloadEntries();
+}
+
+void KNSCore::Engine::fetchEntryById(const QString& id)
+{
+    m_searchTimer->stop();
+    m_currentRequest = KNSCore::Provider::SearchRequest(KNSCore::Provider::Newest, KNSCore::Provider::ExactEntryId, id);
+
+    EntryInternal::List cache = m_cache->requestFromCache(m_currentRequest);
+    if (!cache.isEmpty()) {
+        reloadEntries();
+    } else {
+        m_searchTimer->start();
+    }
+}
+
 void Engine::setSearchTerm(const QString &searchString)
 {
     m_searchTimer->stop();
@@ -354,7 +369,7 @@ void Engine::slotSearchTimerExpired()
 
 void Engine::requestMoreData()
 {
-    // qDebug() << "Get more data! current page: " << m_currentPage  << " requested: " << m_currentRequest.page;
+    qCDebug(KNEWSTUFFCORE) << "Get more data! current page: " << m_currentPage  << " requested: " << m_currentRequest.page;
 
     if (m_currentPage < m_currentRequest.page) {
         return;
@@ -373,7 +388,7 @@ void Engine::requestData(int page, int pageSize)
 
 void Engine::doRequest()
 {
-    foreach (const QSharedPointer<KNS3::Provider> &p, m_providers) {
+    foreach (const QSharedPointer<KNSCore::Provider> &p, m_providers) {
         if (p->isInitialized()) {
             p->loadEntries(m_currentRequest);
             ++m_numDataJobs;
@@ -382,17 +397,17 @@ void Engine::doRequest()
     }
 }
 
-void Engine::install(KNS3::EntryInternal entry, int linkId)
+void Engine::install(KNSCore::EntryInternal entry, int linkId)
 {
-    if (entry.status() == Entry::Updateable) {
-        entry.setStatus(Entry::Updating);
+    if (entry.status() == KNS3::Entry::Updateable) {
+        entry.setStatus(KNS3::Entry::Updating);
     } else  {
-        entry.setStatus(Entry::Installing);
+        entry.setStatus(KNS3::Entry::Installing);
     }
     emit signalEntryChanged(entry);
 
-    // qDebug() << "Install " << entry.name()
-    //    << " from: " << entry.providerId();
+    qCDebug(KNEWSTUFFCORE) << "Install " << entry.name()
+       << " from: " << entry.providerId();
     QSharedPointer<Provider> p = m_providers.value(entry.providerId());
     if (p) {
         p->loadPayloadLink(entry, linkId);
@@ -414,66 +429,66 @@ void Engine::slotInstallationFailed(const QString &message)
     emit signalError(message);
 }
 
-void Engine::slotEntryDetailsLoaded(const KNS3::EntryInternal &entry)
+void Engine::slotEntryDetailsLoaded(const KNSCore::EntryInternal &entry)
 {
     emit signalEntryDetailsLoaded(entry);
 }
 
-void Engine::downloadLinkLoaded(const KNS3::EntryInternal &entry)
+void Engine::downloadLinkLoaded(const KNSCore::EntryInternal &entry)
 {
     m_installation->install(entry);
 }
 
-void Engine::uninstall(KNS3::EntryInternal entry)
+void Engine::uninstall(KNSCore::EntryInternal entry)
 {
-    KNS3::EntryInternal::List list = m_cache->registryForProvider(entry.providerId());
+    KNSCore::EntryInternal::List list = m_cache->registryForProvider(entry.providerId());
     //we have to use the cached entry here, not the entry from the provider
     //since that does not contain the list of installed files
-    KNS3::EntryInternal actualEntryForUninstall;
-    foreach (const KNS3::EntryInternal &eInt, list) {
+    KNSCore::EntryInternal actualEntryForUninstall;
+    foreach (const KNSCore::EntryInternal &eInt, list) {
         if (eInt.uniqueId() == entry.uniqueId()) {
             actualEntryForUninstall = eInt;
             break;
         }
     }
     if (!actualEntryForUninstall.isValid()) {
-        qDebug() << "could not find a cached entry with following id:" << entry.uniqueId() <<
+        qCDebug(KNEWSTUFFCORE) << "could not find a cached entry with following id:" << entry.uniqueId() <<
                  " ->  using the non-cached version";
         return;
     }
 
-    entry.setStatus(Entry::Installing);
-    actualEntryForUninstall.setStatus(Entry::Installing);
+    entry.setStatus(KNS3::Entry::Installing);
+    actualEntryForUninstall.setStatus(KNS3::Entry::Installing);
     emit signalEntryChanged(entry);
 
-    //qDebug() << "about to uninstall entry " << entry.uniqueId();
+    qCDebug(KNEWSTUFFCORE) << "about to uninstall entry " << entry.uniqueId();
     // FIXME: change the status?
     m_installation->uninstall(actualEntryForUninstall);
 
-    entry.setStatus(Entry::Deleted); //status for actual entry gets set in m_installation->uninstall()
+    entry.setStatus(KNS3::Entry::Deleted); //status for actual entry gets set in m_installation->uninstall()
     emit signalEntryChanged(entry);
 
 }
 
-void Engine::loadDetails(const KNS3::EntryInternal &entry)
+void Engine::loadDetails(const KNSCore::EntryInternal &entry)
 {
     QSharedPointer<Provider> p = m_providers.value(entry.providerId());
     p->loadEntryDetails(entry);
 }
 
-void Engine::loadPreview(const KNS3::EntryInternal &entry, EntryInternal::PreviewType type)
+void Engine::loadPreview(const KNSCore::EntryInternal &entry, EntryInternal::PreviewType type)
 {
-    // qDebug() << "START  preview: " << entry.name() << type;
+    qCDebug(KNEWSTUFFCORE) << "START  preview: " << entry.name() << type;
     ImageLoader *l = new ImageLoader(entry, type, this);
-    connect(l, SIGNAL(signalPreviewLoaded(KNS3::EntryInternal,KNS3::EntryInternal::PreviewType)), this, SLOT(slotPreviewLoaded(KNS3::EntryInternal,KNS3::EntryInternal::PreviewType)));
+    connect(l, &ImageLoader::signalPreviewLoaded, this, &Engine::slotPreviewLoaded);
     l->start();
     ++m_numPictureJobs;
     updateStatus();
 }
 
-void Engine::slotPreviewLoaded(const KNS3::EntryInternal &entry, EntryInternal::PreviewType type)
+void Engine::slotPreviewLoaded(const KNSCore::EntryInternal &entry, EntryInternal::PreviewType type)
 {
-    // qDebug() << "FINISH preview: " << entry.name() << type;
+    qCDebug(KNEWSTUFFCORE) << "FINISH preview: " << entry.name() << type;
     emit signalEntryPreviewLoaded(entry, type);
     --m_numPictureJobs;
     updateStatus();
@@ -484,10 +499,10 @@ void Engine::contactAuthor(const EntryInternal &entry)
     if (!entry.author().email().isEmpty()) {
         // invoke mail with the address of the author
         QUrl mailUrl;
-        mailUrl.setScheme("mailto");
+        mailUrl.setScheme(QStringLiteral("mailto"));
         mailUrl.setPath(entry.author().email());
         QUrlQuery query;
-        query.addQueryItem("subject", i18n("Re: %1", entry.name()));
+        query.addQueryItem(QStringLiteral("subject"), i18n("Re: %1", entry.name()));
         mailUrl.setQuery(query);
         QDesktopServices::openUrl(mailUrl);
     } else if (!entry.author().homepage().isEmpty()) {
@@ -495,7 +510,7 @@ void Engine::contactAuthor(const EntryInternal &entry)
     }
 }
 
-void Engine::slotEntryChanged(const KNS3::EntryInternal &entry)
+void Engine::slotEntryChanged(const KNSCore::EntryInternal &entry)
 {
     emit signalEntryChanged(entry);
 }
@@ -540,8 +555,16 @@ void Engine::updateStatus()
 void Engine::checkForUpdates()
 {
     foreach (QSharedPointer<Provider> p, m_providers) {
-        Provider::SearchRequest request(KNS3::Provider::Updates);
+        Provider::SearchRequest request(KNSCore::Provider::Newest, KNSCore::Provider::Updates);
         p->loadEntries(request);
     }
 }
 
+void KNSCore::Engine::checkForInstalled()
+{
+    foreach (QSharedPointer<Provider> p, m_providers) {
+        Provider::SearchRequest request(KNSCore::Provider::Newest, KNSCore::Provider::Installed);
+        request.page = 0;
+        p->loadEntries(request);
+    }
+}
